import { createApi } from "@reduxjs/toolkit/query/react";
import { baseQueryWithReauth } from "@src/infrastructure/apis/utilities";
import { Todo } from "@src/core/entities/todo";

export const todosApi = createApi({
  reducerPath: "api",
  tagTypes: ["todo"],
  baseQuery: baseQueryWithReauth,
  endpoints: (build) => ({
    getTodoById: build.query<Todo, string>({
      query: (id) => `api/todos/${id}`,
      providesTags: (result, error, id) => [{ type: "todo", id }],
    }),

    getAllTodos: build.query<Todo[], void>({
      query: () => ({ url: `api/todos` }),
      providesTags: (result) =>
        result
          ? [
              ...result.map(({ id }) => ({ type: "todo" as const, id })),
              { type: "todo", id: "LIST" },
            ]
          : [],
    }),

    getActiveTodos: build.query<Todo[], void>({
      query: () => ({
        url: `api/todos`,
        params: {
          onlyActive: true,
        },
      }),
      providesTags: (result) =>
        result
          ? [
              ...result.map(({ id }) => ({ type: "todo" as const, id })),
              { type: "todo", id: "LIST" },
            ]
          : [],
    }),

    createTodo: build.mutation<string, Partial<Todo>>({
      query(body) {
        return {
          url: "/api/todos",
          method: "POST",
          body,
        };
      },
      invalidatesTags: (result, error, id) => [{ type: "todo", id: "LIST" }],
    }),

    replaceTodo: build.mutation<string, Partial<Todo>>({
      query(data) {
        const { id, ...body } = data;
        return {
          url: `/api/todos/${id}`,
          method: "PUT",
          body,
        };
      },
      invalidatesTags: (result, error, { id }) => [{ type: "todo", id }],
      async onQueryStarted({ id, ...body }, { dispatch, queryFulfilled }) {
        const replaceResult = dispatch(
          todosApi.util.updateQueryData("getTodoById", id, (draft) => {
            Object.assign(draft, body);
          })
        );

        try {
          await queryFulfilled;
        } catch {
          replaceResult.undo();
          dispatch(todosApi.util.invalidateTags([{ type: "todo", id }]));
        }
      },
    }),
  }),
});
